"""Contact model"""

from hubspyt.models.base_model import BaseModel

class Contact(BaseModel):
    """Wraps a contact entity from HubSpot's API"""

    def __init__(
            self,
            vid=None,
            email=None,
            first_name=None,
            last_name=None,
            company=None,
    ):
        self.vid = vid
        self.email = email
        self.first_name = first_name
        self.last_name = last_name
        self.company = company

    def get_update(self):
        """Gets instance for update to provide to HubSpot"""

        vid = self.vid
        email = self.email
        first_name = self.first_name
        last_name = self.last_name
        company = self.company

        instance = {
            'vid': vid,
            'email': email,
            'properties': [
                {
                    'property': 'firstname',
                    'value': first_name,
                },
                {
                    'property': 'lastname',
                    'value': last_name,
                },
                {
                    'property': 'company',
                    'value': company,
                },
            ],
        }

        return instance



"""HubSpot class"""

from hubspyt.contacts import Contacts

class HubSpot(object):
    """HubSpot's API"""

    def __init__(self, api_key):
        """Set API key"""

        self.api_key = api_key
        self.contacts = Contacts(self.api_key)



"""Contacts API endpoint wrapper"""

import requests

from hubspyt.base_wrapper import BaseWrapper
from hubspyt.models.contact import Contact

class Contacts(BaseWrapper):
    """Wrapper for the contacts endpoint"""

    def _contacts_url(self, addon):
        """Adds an addon URL to the base Contacts url"""

        contacts_url = self.contacts_base_url

        url = '{contacts_url}{addon}'.format(contacts_url=contacts_url, addon=addon)

        return url

    def all(self, count=None):
        """Fetches all current contacts with an optional count"""

        contacts_all_url = self.contacts_all_url
        contact_list = []

        if count is None:
            has_more = True
            vid_offset = None

            while has_more is True:
                url = self._get_url(
                    self._contacts_url(contacts_all_url),
                    [
                        {'name': 'count', 'value': 250},
                        {'name': 'vidOffset', 'value': vid_offset},
                    ],
                )

                response = requests.get(url).json()
                contacts = response.get('contacts')

                if response.get('has-more') is False:
                    has_more = False

                vid_offset = response.get('vid-offset')

                print(vid_offset)

                for item in contacts:
                    vid = item.get('vid')
                    first_name = None
                    last_name = None
                    company = None

                    if item.get('properties').get('firstname'):
                        first_name = item.get('properties').get('firstname').get('value')

                    if item.get('properties').get('lastname'):
                        last_name = item.get('properties').get('lastname').get('value')

                    if item.get('properties').get('company'):
                        company = item.get('properties').get('company').get('value')

                    contact = Contact(
                        vid=vid,
                        first_name=first_name,
                        last_name=last_name,
                        company=company,
                    )

                    contact_list.append(contact)
        else:
            url = self._get_url(self._contacts_url(contacts_all_url))

            response = requests.get(url).json()
            contacts = response.get('contacts')

            for item in contacts:
                vid = item.get('vid')
                first_name = None
                last_name = None
                company = None

                if item.get('properties').get('firstname'):
                    first_name = item.get('properties').get('firstname').get('value')

                if item.get('properties').get('lastname'):
                    last_name = item.get('properties').get('lastname').get('value')

                if item.get('properties').get('company'):
                    company = item.get('properties').get('company').get('value')

                contact = Contact(
                    vid=vid,
                    first_name=first_name,
                    last_name=last_name,
                    company=company,
                )

                contact_list.append(contact)

        return contact_list

    def batch_modify(self, modified):
        """Send HubSpot's API modified/created contacts"""

        contacts_batch_url = self.contacts_batch_url
        url = self._get_url(self._contacts_url(contacts_batch_url))

        modified_list = []

        for contact in modified:
            modified_list.append(contact.get_update())

        response = requests.post(url, json=modified_list)
        status = response.status_code

        if status == 202:
            return True

        print(status, response.json())

        return False



"""Base API endpoint class"""

class BaseWrapper(object):
    """Provides a baseline for the API endpoint wrappers"""

    def __init__(self, api_key):
        """Set API key"""

        self.api_key = api_key
        self.base_url = 'https://api.hubapi.com'
        self.contacts_base_url = '/contacts/v1'
        self.contacts_all_url = '/lists/all/contacts/all'
        self.contacts_batch_url = '/contact/batch'

    def _get_url(self, endpoint, params=None):
        """Constructs a url with a given endpoint and params"""

        base_url = self.base_url
        api_key = self.api_key
        api_key_param = '?hapikey={api_key}'.format(api_key=api_key)

        url = '{base_url}{endpoint}'.format(base_url=base_url, endpoint=endpoint)
        url += api_key_param

        if params is not None:
            for param in params:
                if param.get('value') is not None:
                    url += '&{name}={value}'.format(
                        name=param.get('name'),
                        value=param.get('value'),
                    )

        return url



from setuptools import setup

setup(
    name='hubspyt',
    packages=['hubspyt', 'hubspyt.models'],
    version='0.2',
    description='HubSpot API Wrapper',
    author='Jeff Allen',
    author_email='jeff.allen127@gmail.com',
    url='https://github.com/MailValet/hubspyt',
    download_url='https://github.com/MailValet/hubspyt/archive/master.zip',
    install_requires=['requests'],
)
